package fi.er.webService;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import fi.cas.opplugin.TypeUtils;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiMapping;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author zf
 * 接收九恒星银行账户，并生成金融机构
 */
@ApiController(value = "account", desc = "自定义API 银行账户同步")
@ApiMapping(value = "SaveAccountApi")
public class SaveAccountApi implements Serializable {
    protected static final Long DEFAULT_ORG_ID = 100000L;

    /**
     * @description 行名行号同步接口
     * @param data
     * @return CustomApiResult
     */
    @ApiPostMapping(value = "/save", desc = "同步银行账户")
    public CustomApiResultForMy<List<AccountSynReturnModel>> saveAccount (@ApiParam(value = "九恒星银行账户") JSONArray data) {
        StringBuilder resultString = new StringBuilder();
        List<AccountSynReturnModel> returnList = new ArrayList<>();
        Boolean result = true;

        DynamicObject rmbinfo = BusinessDataServiceHelper.loadSingleFromCache(1L,"bd_currency");
        //默认的国家
        DynamicObject defaultgj = BusinessDataServiceHelper.loadSingleFromCache("1000001", "bd_country");

        RequestContext rc = RequestContext.get();
        DynamicObject user = BusinessDataServiceHelper.loadSingleFromCache(rc.getCurrUserId(), "bos_user");
        DynamicObject defaultOrg = BusinessDataServiceHelper.loadSingle(DEFAULT_ORG_ID, "bos_org");
        //默认金融机构类别
        DynamicObject defaultjrjgtypeinfo = BusinessDataServiceHelper.loadSingleFromCache(615763944224700416L,"bd_finorgtype");

        //默认银行类别
        QFilter typecodeFilter = new QFilter("type_code.number","=","BC-9999");
        DynamicObject defaulttypecodeinfo =BusinessDataServiceHelper.loadSingleFromCache("bd_bankcgsetting",new QFilter[]{typecodeFilter});


        //循环传入的数据
        for (int i = 0; i < data.size(); i++) {
            JSONObject eobject = data.getJSONObject(i);
            AccountSynReturnModel contractSynReturnModel = syncAccount(eobject,rmbinfo,defaultjrjgtypeinfo,defaultgj,defaulttypecodeinfo,defaultOrg,user);
            if ("500".equals(contractSynReturnModel.getCode())){
                result = false;
            }
            returnList.add(contractSynReturnModel);
        }
        return result == true
                ? CustomApiResultForMy.success2(returnList)
                : CustomApiResultForMy.fail(returnList);
    }


    /**
     * @description 保存行名行号
     * @param jsonObject
     */
    private AccountSynReturnModel syncAccount(JSONObject jsonObject,DynamicObject rmbinfo, DynamicObject defaultjrjgtypeinfo,DynamicObject defaultgj,DynamicObject defaulttypecodeinfo, DynamicObject defaultOrg,DynamicObject user) {
        AccountSynReturnModel contractSynReturnModel = new AccountSynReturnModel();
        Date curDate = new Date();
        String number = jsonObject.get("accountnumber")+"";
        String name = jsonObject.get("accountname")+"";
        if ("".equals(number)||"".equals(name)) {
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("账户名和银行账号不能为空");
            return contractSynReturnModel;
        }

        contractSynReturnModel.setNumber(number);
        contractSynReturnModel.setName(name);

        //查询是否存在，存在则返回。不存在更新
        QFilter qFilter = new QFilter("number","=",number);
        DynamicObject accountinfo =  BusinessDataServiceHelper.loadSingle("bd_accountbanks",new QFilter[]{qFilter});
        if(accountinfo!=null){
            //已存在
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("该银行账号已存在");
            return contractSynReturnModel;
        }
        String orgname = jsonObject.get("org")+"";
        if ("".equals(orgname)) {
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("法人主体不能为空");
            return contractSynReturnModel;
        }
        //查询组织是否存在
        QFilter orgqFilter = new QFilter("name","=",orgname);
        DynamicObject orginfo =BusinessDataServiceHelper.loadSingleFromCache("bos_org",new QFilter[]{orgqFilter});
        if(orginfo==null){
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("没有找到名称为："+orgname+"的组织");
            return contractSynReturnModel;
        }

        //账户类型
        String acctstyle = jsonObject.get("acctstyle")+"";
        if ("".equals(acctstyle)) {
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("账户类型不能为空");
            return contractSynReturnModel;
        }
        //opendate开户日期
        String opendate = jsonObject.get("opendate")+"";
        if ("".equals(opendate)) {
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("开户日期不能为空");
            return contractSynReturnModel;
        }


        //开户行行号
        String banknumber = jsonObject.get("banknumber")+"";
        if ("".equals(banknumber)) {
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("开户行行号不能为空");
            return contractSynReturnModel;
        }
        //查询行名行号
        QFilter bankqFilter = new QFilter("number","=",banknumber);
        DynamicObject bankinfo =BusinessDataServiceHelper.loadSingleFromCache("bd_bebank",new QFilter[]{bankqFilter});
        if (bankinfo==null) {
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("开户行行号"+banknumber+"不存在");
            return contractSynReturnModel;
        }
        //通过开户行行号找到金融机构，没有就新增一个
        QFilter jrjgqFilter = new QFilter("bebank.number","=",banknumber);
        DynamicObject jrjginfo =BusinessDataServiceHelper.loadSingleFromCache("bd_finorginfo",new QFilter[]{jrjgqFilter});
        if (jrjginfo==null) {
            jrjginfo = BusinessDataServiceHelper.newDynamicObject("bd_finorginfo");
            jrjginfo.set("finorgtype",defaultjrjgtypeinfo);
            jrjginfo.set("number",banknumber);
            jrjginfo.set("bebank",bankinfo);
            jrjginfo.set("name",bankinfo.get("name")+"");
            //根据行号前3位找到银行类别，找不到就其他
            String typecode = banknumber.substring(0,3);
            QFilter typecodeFilter = new QFilter("type_code.number","=",typecode);
            DynamicObject typecodeinfo =BusinessDataServiceHelper.loadSingleFromCache("bd_bankcgsetting",new QFilter[]{typecodeFilter});
            if(typecodeinfo==null){
                jrjginfo.set("bank_cate",defaulttypecodeinfo);
            }else{
                jrjginfo.set("bank_cate",typecodeinfo);
            }
            //国家
            jrjginfo.set("country",defaultgj);
            //联行号
            jrjginfo.set("union_number",banknumber);


            jrjginfo.set("creator", user);
            jrjginfo.set("createtime", curDate);
            jrjginfo.set("enable",1);
            jrjginfo.set("status","C");
            //执行保存
            StringBuilder err = new StringBuilder();
            OperationResult result = SaveServiceHelper.saveOperate("bd_finorginfo", new DynamicObject[]{jrjginfo}, OperateOption.create());
            if (result.getSuccessPkIds().size() > 0) {
                jrjginfo.set("id",result.getSuccessPkIds().get(0));
            }else{
                for (int k = 0; k < result.getAllErrorOrValidateInfo().size(); k++) {
                    String message = result.getAllErrorOrValidateInfo().get(k).getMessage();
                    err.append("/").append(message);
                }
                contractSynReturnModel.setCode("500");
                contractSynReturnModel.setMessage("金融机构保存失败："+err.toString());
                return contractSynReturnModel;
            }
        }




        accountinfo =BusinessDataServiceHelper.newDynamicObject("bd_accountbanks");
        accountinfo.set("company",orginfo);
        accountinfo.set("openorg",orginfo);
        accountinfo.set("finorgtype","0");
        //开户行,即金融机构
        accountinfo.set("bank",jrjginfo);
        accountinfo.set("bankaccountnumber",number);
        accountinfo.set("name",name);

        //币别，多选基础资料
        DynamicObjectCollection currencyinfos = accountinfo.getDynamicObjectCollection("currency");
        DynamicObject bibiecollinfo = currencyinfos.addNew();
        bibiecollinfo.set("fbasedataid",rmbinfo);
        //默认币种
        accountinfo.set("defaultcurrency",rmbinfo);

        //acctstyle 账户类型
        String acctstylestr = "normal";
        if("基本存款账户".equals(acctstyle)){
            acctstylestr = "basic";
        }else if("临时存款账户".equals(acctstyle)){
            acctstylestr = "temp";
        }else if("专用存款账户".equals(acctstyle)){
            acctstylestr = "spcl";
        }else if("经常项目外汇账户".equals(acctstyle)){
            acctstylestr = "fgn_curr";
        }else if("资本项目外汇账户".equals(acctstyle)){
            acctstylestr = "fng_fin";
        }
        accountinfo.set("acctstyle",acctstylestr);

        accountinfo.set("accttype","in_out");
        accountinfo.set("acctproperty",BusinessDataServiceHelper.loadSingleFromCache(2190699485795992576L,"bd_acctpurpose"));

        try {
            accountinfo.set("opendate", TypeUtils.string2Date(opendate,TypeUtils.DEFAULT_DATE_FORMAT));
        }catch (Exception e){
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("开户日期"+opendate+"格式不对：应为yyyy-MM-dd");
            return contractSynReturnModel;
        }
        accountinfo.set("acctstatus","normal");
        accountinfo.set("elecpaymentlevel","WL01");
        accountinfo.set("manager",user);


        accountinfo.set("enable",1);
        accountinfo.set("status","C");



        accountinfo.set("creator", user);
        accountinfo.set("createorg", defaultOrg);
        accountinfo.set("org", defaultOrg);
        accountinfo.set("createtime", curDate);

        //执行保存
        OperationResult result = SaveServiceHelper.saveOperate("bd_accountbanks", new DynamicObject[]{accountinfo}, OperateOption.create());

        StringBuilder err = new StringBuilder();
        if (result.getSuccessPkIds().size() > 0) {
            if (result.isSuccess()) {
                contractSynReturnModel.setCode("200");
                contractSynReturnModel.setMessage("保存成功");
                return contractSynReturnModel;
            } else {
                for (int k = 0; k < result.getAllErrorOrValidateInfo().size(); k++) {
                    String message = result.getAllErrorOrValidateInfo().get(k).getMessage();
                    err.append("/").append(message);
                }
                contractSynReturnModel.setCode("500");
                contractSynReturnModel.setMessage(err.toString());
                return contractSynReturnModel;
            }
        }else{
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("保存失败");
            return contractSynReturnModel;
        }
    }
}
