package fi.er.webService;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiMapping;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.util.StringUtils;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author zf
 * 接收九恒星行名行号
 */
@ApiController(value = "bank", desc = "自定义API 行名行号同步")
@ApiMapping(value = "saveBankApi")
public class SaveBankApi implements Serializable {
    protected static final Long DEFAULT_ORG_ID = 100000L;

    /**
     * @description 行名行号同步接口
     * @param data
     * @return CustomApiResult
     */
    @ApiPostMapping(value = "/save", desc = "同步行名行号")
    public CustomApiResultForMy<List<BankSynReturnModel>> saveBank (@ApiParam(value = "九恒星行名行号") JSONArray data) {
        StringBuilder resultString = new StringBuilder();
        List<BankSynReturnModel> returnList = new ArrayList<>();
        Boolean result = true;

        //默认的国家
        DynamicObject defaultgj = BusinessDataServiceHelper.loadSingle("1000001", "bd_country");


        RequestContext rc = RequestContext.get();
        DynamicObject user = BusinessDataServiceHelper.loadSingleFromCache(rc.getCurrUserId(), "bos_user");

        //循环传入的数据
        for (int i = 0; i < data.size(); i++) {
            JSONObject eobject = data.getJSONObject(i);
            BankSynReturnModel contractSynReturnModel = syncBank(eobject,defaultgj,user);
            if ("500".equals(contractSynReturnModel.getCode())){
                result = false;
            }
            returnList.add(contractSynReturnModel);
        }
        return result == true
                ? CustomApiResultForMy.success2(returnList)
                : CustomApiResultForMy.fail(returnList);
    }


    /**
     * @description 保存行名行号
     * @param jsonObject
     */
    private BankSynReturnModel syncBank(JSONObject jsonObject,DynamicObject defaultgj ,DynamicObject user) {
        BankSynReturnModel contractSynReturnModel = new BankSynReturnModel();
        String number = jsonObject.get("banknumber")+"";
        String name = jsonObject.get("bankname")+"";
        if ("".equals(number)||"".equals(name)) {
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("名称和编码不能为空");
            return contractSynReturnModel;
        }

        contractSynReturnModel.setNumber(number);
        contractSynReturnModel.setName(name);

        //查询是否存在，存在则返回。不存在更新
        QFilter qFilter = new QFilter("number","=",number);
        DynamicObject bebankinfo =  BusinessDataServiceHelper.loadSingle("bd_bebank",new QFilter[]{qFilter});
        if(bebankinfo!=null){
            //已存在
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("该行号已存在");
            return contractSynReturnModel;
        }

        bebankinfo =BusinessDataServiceHelper.newDynamicObject("bd_bebank");
        bebankinfo.set("number",number);
        bebankinfo.set("name",name);
        bebankinfo.set("country",defaultgj);
        bebankinfo.set("union_number",number);
        bebankinfo.set("provincetxt",jsonObject.get("province")+"");//省份
        bebankinfo.set("citytxt",jsonObject.get("city")+"");//城市
        bebankinfo.set("enable",1);
        bebankinfo.set("status","C");


        Date curDate = new Date();
        bebankinfo.set("creator", user);
        bebankinfo.set("createtime", curDate);

        //执行保存
        OperationResult result = SaveServiceHelper.saveOperate("bd_bebank", new DynamicObject[]{bebankinfo}, OperateOption.create());

        StringBuilder err = new StringBuilder();
        if (result.getSuccessPkIds().size() > 0) {
            if (result.isSuccess()) {
                contractSynReturnModel.setCode("200");
                contractSynReturnModel.setMessage("保存成功");
                return contractSynReturnModel;
            } else {
                for (int k = 0; k < result.getAllErrorOrValidateInfo().size(); k++) {
                    String message = result.getAllErrorOrValidateInfo().get(k).getMessage();
                    err.append("/").append(message);
                }
                contractSynReturnModel.setCode("500");
                contractSynReturnModel.setMessage(err.toString());
                return contractSynReturnModel;
            }
        }else{
            contractSynReturnModel.setCode("500");
            contractSynReturnModel.setMessage("保存失败");
            return contractSynReturnModel;
        }
    }
}
