package nckd.jimin.jyyy.fi.plugin.form;

import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.datamodel.events.BizDataEventArgs;
import kd.bos.entity.property.BasedataProp;
import kd.bos.form.field.BasedataEdit;
import kd.bos.form.field.events.BeforeF7SelectEvent;
import kd.bos.form.field.events.BeforeF7SelectListener;
import kd.bos.list.ListFilterParameter;
import kd.bos.list.ListShowParameter;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.BusinessDataServiceHelper;

import java.util.EventObject;
import java.util.List;
import java.util.Map;

/**
 * 单据标识：对公报销单(nckd_er_publicreimbur_ext)
 * @author wanghaiwu_kd
 * @date 2025/04/28
 */
public class PublicReimburseBillFormPlugin extends AbstractBillPlugIn implements BeforeF7SelectListener {
    @Override
    public void registerListener(EventObject e) {
        super.registerListener(e);

        BasedataEdit expenseitemF7 = (BasedataEdit) this.getControl("expenseitem");
        expenseitemF7.addBeforeF7SelectListener(this);

        BasedataEdit expenseitemF7ext = (BasedataEdit) this.getControl("nckd_expenseitem");
        expenseitemF7ext.addBeforeF7SelectListener(this);
    }

    @Override
    public void beforeF7Select(BeforeF7SelectEvent evt) {
        String property = evt.getProperty().getName();
        String entityId = ((BasedataProp) evt.getProperty()).getBaseEntityId();
        int curRowIndex = evt.getRow();

        if ("expenseitem".equals(property) || "nckd_expenseitem".equals(property)) {
            ListShowParameter showParameter = (ListShowParameter) evt.getFormShowParameter();
            ListFilterParameter filterParam = showParameter.getListFilterParameter();
            List<QFilter> qFilters = filterParam.getQFilters();

            Map<String, Object> customParams = this.getView().getFormShowParameter().getCustomParams();
            if (customParams.get("nckd_viewTypeFilter") != null) {
                QFilter viewTypeFilter = QFilter.fromSerializedString(customParams.get("nckd_viewTypeFilter").toString());

                qFilters.add(viewTypeFilter);
            }

        }
    }

    @Override
    public void afterCreateNewData(EventObject e) {
        super.afterCreateNewData(e);

        Map<String, Object> customParams = this.getView().getFormShowParameter().getCustomParams();
        if(customParams.get("viewType") != null){
            String[] typeStrings = customParams.get("viewType").toString().split(",");

            QFilter qFilter = new QFilter("number", QCP.equals, typeStrings[0]);

            DynamicObject viewTypeInfo = BusinessDataServiceHelper.loadSingle("nckd_payviewtype", qFilter.toArray());
            if(viewTypeInfo != null){
                this.getModel().setValue("nckd_payviewtype", viewTypeInfo);
            }
        }
    }

    @Override
    public void createNewData(BizDataEventArgs e) {
        super.createNewData(e);
    }
}